@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,11 +11,11 @@ import Demo.*;
 
 class SessionPingThread extends Thread
 {
-    SessionPingThread(ChatSessionPrx session)
+    SessionPingThread(ChatSessionPrx session, long timeout)
     {
         _session = session;
 	_destroy = false;
-	_timeout = 20*1000;
+	_timeout = timeout*1000;
     }
 
     synchronized public void
@@ -56,5 +56,5 @@ class SessionPingThread extends Thread
 
     private ChatSessionPrx _session;
     private boolean _destroy;
-    private int _timeout;
+    private long _timeout;
 }
