@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -12,8 +12,18 @@ import Demo.*;
 public class HelloI extends _HelloDisp
 {
     public void
-    sayHello(Ice.Current current)
+    sayHello(int delay, Ice.Current current)
     {
+        if(delay > 0)
+        {
+            try
+            {
+                Thread.currentThread().sleep(delay);
+            }
+            catch(InterruptedException ex1)
+            {
+            }
+        }
         System.out.println("Hello World!");
     }
 
