@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -273,31 +273,6 @@ public final class Connection
 	}
     }
 
-    private final static byte[] _requestHdr =
-    {
-	IceInternal.Protocol.magic[0],
-	IceInternal.Protocol.magic[1],
-	IceInternal.Protocol.magic[2],
-	IceInternal.Protocol.magic[3],
-        IceInternal.Protocol.protocolMajor,
-        IceInternal.Protocol.protocolMinor,
-        IceInternal.Protocol.encodingMajor,
-        IceInternal.Protocol.encodingMinor,
-        IceInternal.Protocol.requestMsg,
-        (byte)0, // Compression status.
-        (byte)0, (byte)0, (byte)0, (byte)0, // Message size (placeholder).
-        (byte)0, (byte)0, (byte)0, (byte)0  // Request ID (placeholder).
-    };
-
-    //
-    // TODO: Should not be a member function of Connection.
-    //
-    public byte[]
-    getRequestHeader()
-    {
-	return _requestHdr;
-    }
-
     public void
     sendRequest(IceInternal.BasicStream os, IceInternal.Outgoing out)
         throws IceInternal.LocalExceptionWrapper
@@ -371,11 +346,11 @@ public final class Connection
 		    readStreamAndParseMessage(os, info);
 		    if(info.invokeNum > 0)
 		    {
-			throw new UnknownMessageException();
+			Ice.Util.throwUnknownMessageException();
 		    }
 		    else if(info.requestId != requestId)
 		    {
-			throw new UnknownRequestIdException();
+			Ice.Util.throwUnknownRequestIdException();
 		    }
 			
 		    out.finished(os);
@@ -473,22 +448,6 @@ public final class Connection
 	}
     }
 
-    private final static byte[] _requestBatchHdr =
-    {
-	IceInternal.Protocol.magic[0],
-	IceInternal.Protocol.magic[1],
-	IceInternal.Protocol.magic[2],
-	IceInternal.Protocol.magic[3],
-        IceInternal.Protocol.protocolMajor,
-        IceInternal.Protocol.protocolMinor,
-        IceInternal.Protocol.encodingMajor,
-        IceInternal.Protocol.encodingMinor,
-        IceInternal.Protocol.requestBatchMsg,
-        0, // Compression status.
-        (byte)0, (byte)0, (byte)0, (byte)0, // Message size (placeholder).
-        (byte)0, (byte)0, (byte)0, (byte)0  // Number of requests in batch (placeholder).
-    };
-
     public synchronized void
     prepareBatchRequest(IceInternal.BasicStream os)
     {
@@ -518,7 +477,7 @@ public final class Connection
         {
 	    try
 	    {
-		_batchStream.writeBlob(_requestBatchHdr);
+		_batchStream.writeBlob(IceInternal.Protocol.requestBatchHdr);
 	    }
 	    catch(LocalException ex)
 	    {
@@ -528,6 +487,7 @@ public final class Connection
         }
 
         _batchStreamInUse = true;
+        _batchMarker = _batchStream.size();
 	_batchStream.swap(os);
 
 	//
@@ -536,63 +496,152 @@ public final class Connection
 	//
     }
 
-    public synchronized void
+    public void
     finishBatchRequest(IceInternal.BasicStream os)
     {
-	//
-	// Get the batch stream back and increment the number of
-	// requests in the batch.
-        //
-	_batchStream.swap(os);
-	++_batchRequestNum;
+        boolean autoflush = false;
+        byte[] lastRequest = null;
 
-	//
-	// Notify about the batch stream not being in use anymore.
-	//
-	if(IceUtil.Debug.ASSERT)
-	{
-	    IceUtil.Debug.Assert(_batchStreamInUse);
-	}
-        _batchStreamInUse = false;
-	notifyAll();
+        synchronized(this)
+        {
+            //
+            // Get the batch stream back.
+            //
+            _batchStream.swap(os);
+
+            if(_batchAutoFlush)
+            {
+                synchronized(_sendMonitor)
+                {
+                    if(_transceiver == null)
+                    {
+                        if(IceUtil.Debug.ASSERT)
+                        {
+                            IceUtil.Debug.Assert(_exception != null);
+                        }
+                        throw _exception; // The exception is immutable at this point.
+                    }
+                    //
+                    // Throw memory limit exception if the first  message added causes us 
+                    // to go over limit. Otherwise put aside the marshalled message that
+                    // caused limit to be exceeded and  rollback stream to the marker. 
+                    //
+                    if(_batchStream.size() > _instance.messageSizeMax())
+                    {
+                        if(_batchRequestNum == 0)
+                        {
+                            resetBatch(true);
+                            throw new Ice.MemoryLimitException();
+                        }
+
+                        lastRequest = new byte[_batchStream.size() - _batchMarker];
+                        IceInternal.ByteBuffer buffer = _batchStream.prepareRead();
+                        buffer.position(_batchMarker);
+                        buffer.get(lastRequest);
+                        _batchStream.resize(_batchMarker, false);
+                        autoflush = true;
+                    }
+                }
+            }
+
+            if(!autoflush)
+            {
+                //
+                // Increment the number of requests in the batch.
+                //
+                ++_batchRequestNum;
+
+                //
+                // Notify about the batch stream not being in use anymore.
+                //
+	        if(IceUtil.Debug.ASSERT)
+	        {
+	            IceUtil.Debug.Assert(_batchStreamInUse);
+	        }
+                _batchStreamInUse = false;
+                notifyAll();
+            }
+        }
+
+        if(autoflush)
+        {
+            //
+            // We have to keep _batchStreamInUse set until after we insert the
+            // saved marshalled data into a new stream.
+            //
+            flushBatchRequestsInternal(true);
+
+            synchronized(this)
+            {
+                //
+                // Throw memory limit exception if the message that caused us to go over
+                // limit causes us to exceed the limit by itself.
+                //
+                if(IceInternal.Protocol.requestBatchHdr.length + lastRequest.length >  _instance.messageSizeMax())
+                {
+                    resetBatch(true);
+                    throw new MemoryLimitException();
+                }
+
+                //
+                // Start a new batch with the last message that caused us to
+                // go over the limit.
+                //
+                try
+                {
+                    _batchStream.writeBlob(IceInternal.Protocol.requestBatchHdr);
+                    _batchStream.writeBlob(lastRequest);
+                }
+                catch(LocalException ex)
+                {
+                    setState(StateClosed, ex);
+                    throw ex;
+                }
+
+                //
+                // Notify that the batch stream not in use anymore.
+                //
+                ++_batchRequestNum;
+                _batchStreamInUse = false;
+                notifyAll();
+            }
+        }
     }
 
     public synchronized void
     abortBatchRequest()
     {
-	//
-	// Destroy and reset the batch stream and batch count. We
-	// cannot save old requests in the batch stream, as they might
-	// be corrupted due to incomplete marshaling.
-	//
-	_batchStream = new IceInternal.BasicStream(_instance);
-	_batchRequestNum = 0;
-
-	//
-	// Notify about the batch stream not being in use anymore.
-	//
-	if(IceUtil.Debug.ASSERT)
-	{
-	    IceUtil.Debug.Assert(_batchStreamInUse);
-	}
-        _batchStreamInUse = false;
-	notifyAll();
+        //
+        // Reset the batch stream. We cannot save old requests
+        // in the batch stream, as they might be corrupted due to
+        // incomplete marshaling.
+        //
+        resetBatch(true);
     }
 
     public void
     flushBatchRequests()
+    {
+        flushBatchRequestsInternal(false);
+    }
+
+    private void
+    flushBatchRequestsInternal(boolean ignoreInUse)
     {
 	synchronized(this)
 	{
-	    while(_batchStreamInUse && _exception == null)
-	    {
-		try
-		{
-		    wait();
-		}
-		catch(InterruptedException ex)
-		{
-		}
+            if(!ignoreInUse)
+            {
+	        while(_batchStreamInUse && _exception == null)
+	        {
+		    try
+		    {
+		        wait();
+		    }
+		    catch(InterruptedException ex)
+		    {
+		    }
+	        }
 	    }
 	    
 	    if(_exception != null)
@@ -678,13 +727,30 @@ public final class Connection
 	    //
 	    // Reset the batch stream, and notify that flushing is over.
 	    //
-	    _batchStream = new IceInternal.BasicStream(_instance);
-	    _batchRequestNum = 0;
-	    _batchStreamInUse = false;
-	    notifyAll();
+            resetBatch(!ignoreInUse);
 	}
     }
 
+    private void
+    resetBatch(boolean resetInUse)
+    {
+        _batchStream = new IceInternal.BasicStream(_instance, _batchAutoFlush);
+        _batchRequestNum = 0;
+
+        //
+        // Notify about the batch stream not being in use anymore.
+        //
+        if(resetInUse)
+        {
+            if(IceUtil.Debug.ASSERT)
+            {
+                IceUtil.Debug.Assert(_batchStreamInUse);
+            }
+            _batchStreamInUse = false;
+            notifyAll();
+        }
+    }
+
     public void
     sendResponse(IceInternal.BasicStream os)
     {
@@ -840,8 +906,8 @@ public final class Connection
         //
         Connection[] connections = new Connection[1];
         connections[0] = this;
-        IceInternal.Reference ref = _instance.referenceFactory().create(ident, new java.util.Hashtable(), "",
-                                                                        IceInternal.Reference.ModeTwoway, connections);
+        IceInternal.Reference ref = 
+            _instance.referenceFactory().create(ident, null, "", IceInternal.Reference.ModeTwoway, connections);
         return _instance.proxyFactory().referenceToProxy(ref);
     }
 
@@ -871,17 +937,19 @@ public final class Connection
 	_desc = transceiver.toString();
         _type = transceiver.type();
         _endpoint = endpoint;
-        _logger = instance.logger(); // Cached for better performance.
+        _logger = instance.initializationData().logger; // Cached for better performance.
         _traceLevels = instance.traceLevels(); // Cached for better performance.
-	_warn = _instance.properties().getPropertyAsInt("Ice.Warn.Connections") > 0 ? true : false;
+	_warn = _instance.initializationData().properties.getPropertyAsInt("Ice.Warn.Connections") > 0 ? true : false;
         _nextRequestId = 1;
-        _batchStream = new IceInternal.BasicStream(instance);
+        _batchAutoFlush = _instance.initializationData().properties.getPropertyAsIntWithDefault(
+                "Ice.BatchAutoFlush", 1) > 0 ? true : false;
+        _batchStream = new IceInternal.BasicStream(instance, _batchAutoFlush);
 	_batchStreamInUse = false;
 	_batchRequestNum = 0;
         _dispatchCount = 0;
         _state = StateNotValidated;
 	_stateTime = System.currentTimeMillis();
-	_blocking = _instance.properties().getPropertyAsInt("Ice.Blocking") > 0 && adapter == null;
+	_blocking = _instance.initializationData().properties.getPropertyAsInt("Ice.Blocking") > 0 && adapter == null;
 	_stream = new IceInternal.BasicStream(_instance);
 	_in = new IceInternal.Incoming(_instance, this, _stream, adapter);
 
@@ -905,7 +973,7 @@ public final class Connection
 	        ex.printStackTrace();
 	        String s = "cannot create thread for connection:\n";;
 	        s += ex.toString();
-	        _instance.logger().error(s);
+	        _logger.error(s);
 	    
 	        try
 	        {
@@ -1028,42 +1096,30 @@ public final class Connection
 	        if(m[0] != IceInternal.Protocol.magic[0] || m[1] != IceInternal.Protocol.magic[1] ||
 	           m[2] != IceInternal.Protocol.magic[2] || m[3] != IceInternal.Protocol.magic[3])
 	        {
-	            BadMagicException ex = new BadMagicException();
-	    	    ex.badMagic = m;
-	    	    throw ex;
+                    Ice.Util.throwBadMagicException(m);
 	        }
 	        byte pMajor = is.readByte();
 	        byte pMinor = is.readByte();
 	        if(pMajor != IceInternal.Protocol.protocolMajor)
 	        {
-	    	    UnsupportedProtocolException e = new UnsupportedProtocolException();
-	    	    e.badMajor = pMajor < 0 ? pMajor + 255 : pMajor;
-	    	    e.badMinor = pMinor < 0 ? pMinor + 255 : pMinor;
-	    	    e.major = IceInternal.Protocol.protocolMajor;
-	    	    e.minor = IceInternal.Protocol.protocolMinor;
-	    	    throw e;
+                    Ice.Util.throwUnsupportedProtocolException(pMajor, pMinor);
 	        }
 	        byte eMajor = is.readByte();
 	        byte eMinor = is.readByte();
 	        if(eMajor != IceInternal.Protocol.encodingMajor)
 	        {
-	    	    UnsupportedEncodingException e = new UnsupportedEncodingException();
-	    	    e.badMajor = eMajor < 0 ? eMajor + 255 : eMajor;
-	    	    e.badMinor = eMinor < 0 ? eMinor + 255 : eMinor;
-	    	    e.major = IceInternal.Protocol.encodingMajor;
-	    	    e.minor = IceInternal.Protocol.encodingMinor;
-	    	    throw e;
+                    Ice.Util.throwUnsupportedEncodingException(eMajor, eMinor);
 	        }
 	        byte messageType = is.readByte();
 	        if(messageType != IceInternal.Protocol.validateConnectionMsg)
 	        {
-	    	    throw new ConnectionNotValidatedException();
+	    	    Ice.Util.throwConnectionNotValidatedException();
 	        }
 	        byte compress = is.readByte(); // Ignore compression status for validate connection.
 	        int size = is.readInt();
 	        if(size != IceInternal.Protocol.headerSize)
 	        {
-	    	    throw new IllegalMessageSizeException();
+	    	    Ice.Util.throwIllegalMessageSizeException();
 	        }
 	        IceInternal.TraceUtil.traceHeader("received validate connection", is, _logger, _traceLevels);
 	    }
@@ -1326,31 +1382,19 @@ public final class Connection
 	if(m[0] != IceInternal.Protocol.magic[0] || m[1] != IceInternal.Protocol.magic[1] ||
 	   m[2] != IceInternal.Protocol.magic[2] || m[3] != IceInternal.Protocol.magic[3])
 	{
-	    BadMagicException ex = new BadMagicException();
-	    ex.badMagic = m;
-	    throw ex;
+            Ice.Util.throwBadMagicException(m);
 	}
 	byte pMajor = stream.readByte();
 	byte pMinor = stream.readByte();
 	if(pMajor != IceInternal.Protocol.protocolMajor)
 	{
-	    UnsupportedProtocolException e = new UnsupportedProtocolException();
-	    e.badMajor = pMajor < 0 ? pMajor + 255 : pMajor;
-	    e.badMinor = pMinor < 0 ? pMinor + 255 : pMinor;
-	    e.major = IceInternal.Protocol.protocolMajor;
-	    e.minor = IceInternal.Protocol.protocolMinor;
-	    throw e;
+            Ice.Util.throwUnsupportedProtocolException(pMajor, pMinor);
 	}
 	byte eMajor = stream.readByte();
 	byte eMinor = stream.readByte();
 	if(eMajor != IceInternal.Protocol.encodingMajor)
 	{
-	    UnsupportedEncodingException e = new UnsupportedEncodingException();
-	    e.badMajor = eMajor < 0 ? eMajor + 255 : eMajor;
-	    e.badMinor = eMinor < 0 ? eMinor + 255 : eMinor;
-	    e.major = IceInternal.Protocol.encodingMajor;
-	    e.minor = IceInternal.Protocol.encodingMinor;
-	    throw e;
+            Ice.Util.throwUnsupportedEncodingException(eMajor, eMinor);
 	}
 	byte messageType = stream.readByte();
 	byte compress = stream.readByte();
@@ -1364,7 +1408,7 @@ public final class Connection
 	int size = stream.readInt();
 	if(size < IceInternal.Protocol.headerSize)
 	{
-	    throw new IllegalMessageSizeException();
+	    Ice.Util.throwIllegalMessageSizeException();
 	}
 	if(size > _instance.messageSizeMax())
 	{
@@ -1420,7 +1464,7 @@ public final class Connection
 	    if(info.invokeNum < 0)
 	    {
 		info.invokeNum = 0;
-		throw new NegativeSizeException();
+		Ice.Util.throwNegativeSizeException();
 	    }
 	    break;
 	}
@@ -1440,26 +1484,11 @@ public final class Connection
 	    IceInternal.TraceUtil.traceHeader("received unexpected message\n" +
 					      "(invalid, closing connection)", stream, _logger,
 					      _traceLevels);
-	    throw new UnknownMessageException();
+	    Ice.Util.throwUnknownMessageException();
 	}
 	}
     }
 
-    private final static byte[] _replyHdr =
-    {
-        IceInternal.Protocol.magic[0],
-        IceInternal.Protocol.magic[1],
-        IceInternal.Protocol.magic[2],
-        IceInternal.Protocol.magic[3],
-        IceInternal.Protocol.protocolMajor,
-        IceInternal.Protocol.protocolMinor,
-        IceInternal.Protocol.encodingMajor,
-        IceInternal.Protocol.encodingMinor,
-        IceInternal.Protocol.replyMsg,
-        (byte)0, // Compression status.
-        (byte)0, (byte)0, (byte)0, (byte)0 // Message size (placeholder).
-    };
-
     public void
     run()
     {
@@ -1564,7 +1593,7 @@ public final class Connection
 				}
 				else
 				{
-				    throw new UnknownRequestIdException();
+				    Ice.Util.throwUnknownRequestIdException();
 				}
 			    }
 			}
@@ -1656,11 +1685,11 @@ public final class Connection
 			// Add the reply header and request id.
 			//
 			IceInternal.BasicStream os = _in.os();
-			os.writeBlob(_replyHdr);
+			os.writeBlob(IceInternal.Protocol.replyHdr);
 			os.writeInt(info.requestId);
 		    }
 		
-		    _in.invoke(response);
+		    _in.invoke(response, info.requestId);
 		}
 	    }
 	    catch(LocalException ex)
@@ -1804,9 +1833,11 @@ public final class Connection
 
     private LocalException _exception;
 
+    private boolean _batchAutoFlush;
     private IceInternal.BasicStream _batchStream;
     private boolean _batchStreamInUse;
     private int _batchRequestNum;
+    private int _batchMarker;
 
     private int _dispatchCount;
 
