@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -38,12 +38,79 @@ public final class ObjectAdapter
 	{
 	    checkForDeactivation();
 	    
-	    if(!_printAdapterReadyDone)
+	    //
+	    // If the one off initializations of the adapter are already
+	    // done, we just need to activate the incoming connection
+	    // factories and we're done.
+	    //
+	    if(_activateOneOffDone)
 	    {
-		locatorInfo = _locatorInfo;
-		printAdapterReady = _instance.properties().getPropertyAsInt("Ice.PrintAdapterReady") > 0;
-		_printAdapterReadyDone = true;
+		final int sz = _incomingConnectionFactories.size();
+		java.util.Enumeration e = _incomingConnectionFactories.elements();
+		while(e.hasMoreElements())
+		{
+		    IceInternal.IncomingConnectionFactory factory =
+			(IceInternal.IncomingConnectionFactory)e.nextElement();
+		    factory.activate();
+		}
+		return;
 	    }
+
+	    //
+	    // One off initializations of the adapter: update the
+	    // locator registry and print the "adapter ready"
+	    // message. We set the _waitForActivate flag to prevent
+	    // deactivation from other threads while these one off
+	    // initializations are done.
+	    //
+	    _waitForActivate = true;
+	    
+	    locatorInfo = _locatorInfo;
+	    final Properties properties = _instance.initializationData().properties;
+	    printAdapterReady = properties.getPropertyAsInt("Ice.PrintAdapterReady") > 0;
+	}
+
+	try
+	{
+	    Ice.Identity dummy = new Ice.Identity();
+	    dummy.name = "dummy";
+	    updateLocatorRegistry(locatorInfo, createDirectProxy(dummy));
+	}
+	catch(Ice.LocalException ex)
+	{
+	    //
+	    // If we couldn't update the locator registry, we let the
+	    // exception go through and don't activate the adapter to
+	    // allow to user code to retry activating the adapter
+	    // later.
+	    //
+	    synchronized(this)
+	    {
+		_waitForActivate = false;
+		notifyAll();
+	    }
+	    throw ex;
+	}
+
+	if(printAdapterReady)
+	{
+	    System.out.println(_name + " ready");
+	}
+
+	synchronized(this)
+	{
+	    if(IceUtil.Debug.ASSERT)
+	    {
+		IceUtil.Debug.Assert(!_deactivated); // Not possible if _waitForActivate = true;
+	    }
+	    
+	    //
+	    // Signal threads waiting for the activation.
+	    //
+	    _waitForActivate = false;
+	    notifyAll();
+
+	    _activateOneOffDone = true;
 	    
 	    final int sz = _incomingConnectionFactories.size();
 	    java.util.Enumeration e = _incomingConnectionFactories.elements();
@@ -53,77 +120,7 @@ public final class ObjectAdapter
 		    (IceInternal.IncomingConnectionFactory)e.nextElement();
 		factory.activate();
 	    }
-	}
-
-	if(_id.length() > 0)
-	{
-	    //
-	    // We must get and call on the locator registry outside the thread
-	    // synchronization to avoid deadlocks. (we can't make remote calls
-	    // within the OA synchronization because the remote call will
-	    // indirectly call isLocal() on this OA with the OA factory
-	    // locked).
-	    //
-
-	    LocatorRegistryPrx locatorRegistry = null;
-	    if(locatorInfo != null)
-	    {
-		//
-		// TODO: This might throw if we can't connect to the
-		// locator. Shall we raise a special exception for the
-		// activate operation instead of a non obvious network
-		// exception?
-		//
-		locatorRegistry = _locatorInfo.getLocatorRegistry();
-	    }
-	    
-	    if(locatorRegistry != null)
-	    {
-		try
-		{
-		    Identity ident = new Identity();
-		    ident.category = "";
-		    ident.name = "dummy";
-		    if(_replicaGroupId.length() == 0)
-		    {
-			locatorRegistry.setAdapterDirectProxy(_id, createDirectProxy(ident));
-		    }
-		    else
-		    {
-			locatorRegistry.setReplicatedAdapterDirectProxy(_id, _replicaGroupId,createDirectProxy(ident));
-		    }
-		}
-		catch(ObjectAdapterDeactivatedException ex)
-		{
-		    // IGNORE: The object adapter is already inactive.
-		}
-		catch(AdapterNotFoundException ex)
-		{
-		    NotRegisteredException ex1 = new NotRegisteredException();
-		    ex1.kindOfObject = "object adapter";
-		    ex1.id = _id;
-		    throw ex1;
-		}
-		catch(InvalidReplicaGroupIdException ex)
-		{
-		    NotRegisteredException ex1 = new NotRegisteredException();
-		    ex1.kindOfObject = "replica group";
-		    ex1.id = _replicaGroupId;
-		    throw ex1;
-		}
-		catch(AdapterAlreadyActiveException ex)
-		{
-		    ObjectAdapterIdInUseException ex1 = new ObjectAdapterIdInUseException();
-		    ex1.id = _id;
-		    throw ex1;
-		}
-	    }
-	}
-
-	if(printAdapterReady)
-	{
-	    System.out.println(_name + " ready");
-	}
+	}	
     }
 
     public synchronized void
@@ -159,6 +156,7 @@ public final class ObjectAdapter
     {
 	java.util.Vector incomingConnectionFactories;
 	IceInternal.OutgoingConnectionFactory outgoingConnectionFactory;
+	IceInternal.LocatorInfo locatorInfo;
 
 	synchronized(this)
 	{
@@ -171,6 +169,34 @@ public final class ObjectAdapter
 		return;
 	    }
 
+	    //
+	    //
+	    // Wait for activation to complete. This is necessary to not 
+	    // get out of order locator updates.
+	    //
+	    while(_waitForActivate)
+	    {
+		try
+		{
+		    wait();
+		}
+		catch(InterruptedException ex)
+		{
+		}
+	    }
+
+            if(_routerInfo != null)
+            {
+                //
+                // Remove entry from the router manager.
+                //
+                _instance.routerManager().erase(_routerInfo.getRouter());
+
+                //
+                //  Clear this object adapter with the router.
+                //
+                _routerInfo.setAdapter(null);
+            }
 	    
 	    //
 	    // No clone call with J2ME.
@@ -183,12 +209,25 @@ public final class ObjectAdapter
                 incomingConnectionFactories.addElement(e.nextElement());
             }
 	    outgoingConnectionFactory = _instance.outgoingConnectionFactory();
+	    locatorInfo = _locatorInfo;
 	    
 	    _deactivated = true;
 
 	    notifyAll();
 	}
 
+	try
+	{
+	    updateLocatorRegistry(locatorInfo, null);
+	}
+	catch(Ice.LocalException ex)
+	{
+	    //
+	    // We can't throw exceptions in deactivate so we ignore
+	    // failures to update the locator registry.
+	    //
+	}
+
 	//
 	// Must be called outside the thread synchronization, because
 	// Connection::destroy() might block when sending a
@@ -213,86 +252,132 @@ public final class ObjectAdapter
     public void
     waitForDeactivate()
     {
-	synchronized(this)
-	{
-	    //
-	    // First we wait for deactivation of the adapter itself, and
-	    // for the return of all direct method calls using this
-	    // adapter.
-	    //
-	    while(!_deactivated || _directCount > 0)
-	    {
-		try
-		{
-		    wait();
-		}
-		catch(InterruptedException ex)
-		{
-		}
-	    }
-	    
-	    //
-	    // If some other thread is currently deactivating, we wait
-	    // until this thread is finished.
-	    //
-	    while(_waitForDeactivate)
-	    {
-		try
-		{
-		    wait();
-		}
-		catch(InterruptedException ex)
-		{
-		}
-	    }
-	    _waitForDeactivate = true;
-	}
-	
-	//
-	// Now we wait for until all incoming connection factories are
-	// finished.
-	//
-	if(_incomingConnectionFactories != null)
-	{
-	    java.util.Enumeration e = _incomingConnectionFactories.elements();
-	    while(e.hasMoreElements())
-	    {
-		IceInternal.IncomingConnectionFactory factory =
-		    (IceInternal.IncomingConnectionFactory)e.nextElement();
-		factory.waitUntilFinished();
-	    }
-	}
-	
-	//
-	// Now it's also time to clean up our servants and servant
-	// locators.
-	//
-	if(_servantManager != null)
-	{
-	    _servantManager.destroy();
-	}
-	
-	synchronized(this)
-	{
-	    //
-	    // Signal that waiting is complete.
-	    //
-	    _waitForDeactivate = false;
-	    notifyAll();
+        IceInternal.IncomingConnectionFactory[] incomingConnectionFactories;
 
-	    //
-	    // We're done, now we can throw away all incoming connection
-	    // factories.
-	    //
-	    _incomingConnectionFactories = null;
-	    
-	    //
-	    // Remove object references (some of them cyclic).
-	    //
-	    _instance = null;
-	    _servantManager = null;
-	    _communicator = null;
-	}
+        synchronized(this)
+        {
+            if(_destroyed)
+            {
+                return;
+            }
+
+            //
+            // Wait for deactivation of the adapter itself, and
+            // for the return of all direct method calls using this
+            // adapter.
+            //
+            while(!_deactivated || _directCount > 0)
+            {
+                try
+                {
+                    wait();
+                }
+                catch(InterruptedException ex)
+                {
+                }
+            }
+
+            incomingConnectionFactories = new IceInternal.IncomingConnectionFactory[_incomingConnectionFactories.size()];
+            _incomingConnectionFactories.copyInto(incomingConnectionFactories);
+        }
+
+        //
+        // Now we wait for until all incoming connection factories are
+        // finished.
+        //
+        for(int i = 0; i < incomingConnectionFactories.length; ++i)
+        {
+            incomingConnectionFactories[i].waitUntilFinished();
+        }
+    }
+
+    public synchronized boolean
+    isDeactivated()
+    {
+        return _deactivated;
+    }
+
+    public void
+    destroy()
+    {
+        synchronized(this)
+        {
+            //
+            // Another thread is in the process of destroying the object
+            // adapter. Wait for it to finish.
+            //
+            while(_destroying)
+            {
+                try
+                {
+                    wait();
+                }
+                catch(InterruptedException ex)
+                {
+                }
+            }
+
+            //
+            // Object adpater is already destroyed.
+            //
+            if(_destroyed)
+            {
+                return;
+            }
+
+            _destroying = true;
+        }
+
+        //
+        // Deactivate and wait for completion.
+        //
+        deactivate();
+        waitForDeactivate();
+
+        //
+        // Now it's also time to clean up our servants and servant
+        // locators.
+        //
+        _servantManager.destroy();
+
+        IceInternal.ObjectAdapterFactory objectAdapterFactory;
+
+        synchronized(this)
+        {
+            //
+            // Signal that destroying is complete.
+            //
+            _destroying = false;
+            _destroyed = true;
+            notifyAll();
+
+            //
+            // We're done, now we can throw away all incoming connection
+            // factories.
+            //
+            // For compatibility with C#, we set _incomingConnectionFactories
+            // to null so that the finalizer does not invoke methods on objects.
+            //
+            _incomingConnectionFactories = null;
+
+            //
+            // Remove object references (some of them cyclic).
+            //
+            _instance = null;
+            _communicator = null;
+            _routerEndpoints = null;
+            _routerInfo = null;
+            _publishedEndpoints = null;
+            _locatorInfo = null;
+
+            objectAdapterFactory = _objectAdapterFactory;
+            _objectAdapterFactory = null;
+        }
+
+        if(objectAdapterFactory != null)
+        {
+            objectAdapterFactory.removeObjectAdapter(_name);
+        }
     }
 
     public ObjectPrx
@@ -449,125 +534,11 @@ public final class ObjectAdapter
         IceInternal.Endpoint[] endpoints = new IceInternal.Endpoint[0];
         Connection[] arr = new Connection[connections.size()];
         connections.copyInto( arr);
-        IceInternal.Reference ref = _instance.referenceFactory().create(ident, new java.util.Hashtable(), "",
-                                                                        IceInternal.Reference.ModeTwoway, arr);
+        IceInternal.Reference ref = 
+                _instance.referenceFactory().create(ident, null, "", IceInternal.Reference.ModeTwoway, arr);
         return _instance.proxyFactory().referenceToProxy(ref);
     }
 
-    public synchronized void
-    addRouter(RouterPrx router)
-    {
-	checkForDeactivation();
-
-        IceInternal.RouterInfo routerInfo = _instance.routerManager().get(router);
-        if(routerInfo != null)
-        {
-	    _routerInfos.addElement(routerInfo);
-
-            //
-            // Add the router's server proxy endpoints to this object
-            // adapter.
-            //
-            ObjectPrxHelperBase proxy = (ObjectPrxHelperBase)routerInfo.getServerProxy();
-            IceInternal.Endpoint[] endpoints = proxy.__reference().getEndpoints();
-            for(int i = 0; i < endpoints.length; ++i)
-            {
-                _routerEndpoints.addElement(endpoints[i]);
-            }
-	    
-            IceUtil.Arrays.sort(_routerEndpoints); // Must be sorted.
-	    //
-	    // Remove duplicate endpoints, so we have a list of unique
-	    // endpoints.
-	    //
-	    for(int i = 0; i < _routerEndpoints.size()-1; )
-	    {
-                java.lang.Object o1 = _routerEndpoints.elementAt(i);
-                java.lang.Object o2 = _routerEndpoints.elementAt(i + 1);
-                if(o1.equals(o2))
-                {
-                    _routerEndpoints.removeElementAt(i);
-                }
-		else
-		{
-		    ++i;
-		}
-	    }
-
-            //
-            // Associate this object adapter with the router. This way,
-            // new outgoing connections to the router's client proxy will
-            // use this object adapter for callbacks.
-            //
-            routerInfo.setAdapter(this);
-
-            //
-            // Also modify all existing outgoing connections to the
-            // router's client proxy to use this object adapter for
-            // callbacks.
-            //      
-            _instance.outgoingConnectionFactory().setRouterInfo(routerInfo);
-        }
-    }
-
-    public synchronized void
-    removeRouter(RouterPrx router)
-    {
-	checkForDeactivation();
-
-	IceInternal.RouterInfo routerInfo = _instance.routerManager().erase(router);
-	if(routerInfo != null)
-	{
-	    //
-	    // Rebuild the router endpoints from our set of router infos.
-	    //
-	    _routerEndpoints.removeAllElements();
-	    for(int p = 0; p < _routerInfos.size();)
-	    {
-		IceInternal.RouterInfo curr = (IceInternal.RouterInfo)_routerInfos.elementAt(p);
-		if(curr == routerInfo)
-		{
-		    _routerEndpoints.removeElementAt(p);
-		    continue;
-		}
-		ObjectPrxHelperBase proxy = (ObjectPrxHelperBase)routerInfo.getServerProxy();
-		IceInternal.Endpoint[] endpoints = proxy.__reference().getEndpoints();
-		for(int i = 0; i < endpoints.length; ++i)
-		{
-		    _routerEndpoints.addElement(endpoints[i]);
-		}
-		++p;
-	    }
-
-            IceUtil.Arrays.sort(_routerEndpoints); // Must be sorted.
-	    //
-	    // Remove duplicate endpoints, so we have a list of unique
-	    // endpoints.
-	    //
-            for(int i = 0; i < _routerEndpoints.size() - 1; ++i)
-            {
-                java.lang.Object o1 = _routerEndpoints.elementAt(i);
-                java.lang.Object o2 = _routerEndpoints.elementAt(i + 1);
-                if(o1.equals(o2))
-                {
-                    _routerEndpoints.removeElementAt(i);
-                }
-            }
-
-	    //
-	    // Clear this object adapter with the router.
-	    //
-	    routerInfo.setAdapter(null);
-
-	    //
-	    // Also modify all existing outgoing connections to the
-	    // router's client proxy to use this object adapter for
-	    // callbacks.
-	    //	
-	    _instance.outgoingConnectionFactory().setRouterInfo(routerInfo);
-	}
-    }
-
     public synchronized void
     setLocator(LocatorPrx locator)
     {
@@ -576,21 +547,6 @@ public final class ObjectAdapter
 	_locatorInfo = _instance.locatorManager().get(locator);
     }
 
-    public synchronized LocatorPrx
-    getLocator()
-    {
-        checkForDeactivation();
-
-        LocatorPrx locator = null;
-
-        if(_locatorInfo != null)
-        {
-            locator = _locatorInfo.getLocator();
-        }
-
-        return locator;
-    }
-
     public void
     flushBatchRequests()
     {
@@ -646,17 +602,9 @@ public final class ObjectAdapter
     public IceInternal.ServantManager
     getServantManager()
     {	
-	// No mutex lock necessary, _instance is
-	// immutable after creation until it is removed in
-	// waitForDeactivate().
-	
-	// No check for deactivation here!
-	
-	if(IceUtil.Debug.ASSERT)
-	{
-	    IceUtil.Debug.Assert(_instance != null); // Must not be called after waitForDeactivate().
-	}
-
+	//
+	// No mutex lock necessary, _servantManager is immutable.
+	//
 	return _servantManager;
     }
 
@@ -664,48 +612,125 @@ public final class ObjectAdapter
     // Only for use by IceInternal.ObjectAdapterFactory
     //
     public
-    ObjectAdapter(IceInternal.Instance instance, Communicator communicator, String name, String endpointInfo)
+    ObjectAdapter(IceInternal.Instance instance, Communicator communicator, 
+		  IceInternal.ObjectAdapterFactory objectAdapterFactory, String name, String endpointInfo,
+		  RouterPrx router)
     {
 	_deactivated = false;
         _instance = instance;
 	_communicator = communicator;
+	_objectAdapterFactory = objectAdapterFactory;
 	_servantManager = new IceInternal.ServantManager(instance, name);
-	_printAdapterReadyDone = false;
+	_activateOneOffDone = false;
         _name = name;
-	_id = instance.properties().getProperty(name + ".AdapterId");
-	_replicaGroupId = instance.properties().getProperty(name + ".ReplicaGroupId");
+	_id = instance.initializationData().properties.getProperty(name + ".AdapterId");
+	_replicaGroupId = instance.initializationData().properties.getProperty(name + ".ReplicaGroupId");
 	_directCount = 0;
-	_waitForDeactivate = false;
+	_waitForActivate = false;
+	_destroying = false;
+	_destroyed = false;
 	
         try
         {
-	    //
-	    // Parse the endpoints, but don't store them in the adapter.
-	    // The connection factory might change it, for example, to
-	    // fill in the real port number.
-	    //
-	    java.util.Vector endpoints = parseEndpoints(endpointInfo);
-	    for(int i = 0; i < endpoints.size(); ++i)
-	    {
-		IceInternal.Endpoint endp = (IceInternal.Endpoint)endpoints.elementAt(i);
-                _incomingConnectionFactories.addElement(
-		    new IceInternal.IncomingConnectionFactory(instance, endp, this));
+            if(router == null)
+            {
+                String routerStr = _instance.initializationData().properties.getProperty(name + ".Router");
+                if(routerStr.length() > 0)
+                {
+                    router = RouterPrxHelper.uncheckedCast(_instance.proxyFactory().stringToProxy(routerStr));
+                }
             }
+            if(router != null)
+            {
+                _routerInfo = _instance.routerManager().get(router);
+                if(_routerInfo != null)
+                {
+                    //
+                    // Make sure this router is not already registered with another adapter.
+                    //
+                    if(_routerInfo.getAdapter() != null)
+                    {
+                        throw new AlreadyRegisteredException("object adapter with router",
+                                                             _instance.identityToString(router.ice_getIdentity()));
+                    }
 
-	    //
-	    // Parse published endpoints. These are used in proxies
-	    // instead of the connection factory endpoints.
-	    //
-	    String endpts = _instance.properties().getProperty(name + ".PublishedEndpoints");
-	    _publishedEndpoints = parseEndpoints(endpts);
+                    //
+                    // Add the router's server proxy endpoints to this object
+                    // adapter.
+                    //
+                    IceInternal.Endpoint[] endpoints = _routerInfo.getServerEndpoints();
+                    for(int i = 0; i < endpoints.length; ++i)
+                    {
+                        _routerEndpoints.addElement(endpoints[i]);
+                    }
+	    
+                    IceUtil.Arrays.sort(_routerEndpoints); // Must be sorted.
+	            //
+	            // Remove duplicate endpoints, so we have a list of unique
+	            // endpoints.
+	            //
+	            for(int i = 0; i < _routerEndpoints.size()-1; )
+	            {
+                        java.lang.Object o1 = _routerEndpoints.elementAt(i);
+                        java.lang.Object o2 = _routerEndpoints.elementAt(i + 1);
+                        if(o1.equals(o2))
+                        {
+                            _routerEndpoints.removeElementAt(i);
+                        }
+		        else
+		        {
+		            ++i;
+		        }
+	            }
 
-	    String router = _instance.properties().getProperty(name + ".Router");
-	    if(router.length() > 0)
+                    //
+                    // Associate this object adapter with the router. This way,
+                    // new outgoing connections to the router's client proxy will
+                    // use this object adapter for callbacks.
+                    //
+                    _routerInfo.setAdapter(this);
+
+                    //
+                    // Also modify all existing outgoing connections to the
+                    // router's client proxy to use this object adapter for
+                    // callbacks.
+                    //      
+                    _instance.outgoingConnectionFactory().setRouterInfo(_routerInfo);
+                }
+	    }
+	    else
 	    {
-		addRouter(RouterPrxHelper.uncheckedCast(_instance.proxyFactory().stringToProxy(router)));
+	        //
+	        // Parse the endpoints, but don't store them in the adapter.
+	        // The connection factory might change it, for example, to
+	        // fill in the real port number.
+	        //
+	        java.util.Vector endpoints = parseEndpoints(endpointInfo);
+	        for(int i = 0; i < endpoints.size(); ++i)
+	        {
+		    IceInternal.Endpoint endp = (IceInternal.Endpoint)endpoints.elementAt(i);
+                    _incomingConnectionFactories.addElement(
+		        new IceInternal.IncomingConnectionFactory(instance, endp, this));
+                }
+		if(endpoints.size() == 0)
+		{
+		    IceInternal.TraceLevels tl = _instance.traceLevels();
+		    if(tl.network >= 2)
+		    {
+			_instance.initializationData().logger.trace(tl.networkCat,
+			                                            "created adapter `" + name + "' without endpoints");
+		    }
+		}
+
+	        //
+	        // Parse published endpoints. These are used in proxies
+	        // instead of the connection factory endpoints.
+	        //
+	        String endpts = _instance.initializationData().properties.getProperty(name + ".PublishedEndpoints");
+	        _publishedEndpoints = parseEndpoints(endpts);
 	    }
 
-	    String locator = _instance.properties().getProperty(name + ".Locator");
+	    String locator = _instance.initializationData().properties.getProperty(name + ".Locator");
 	    if(locator.length() > 0)
 	    {
 		setLocator(LocatorPrxHelper.uncheckedCast(_instance.proxyFactory().stringToProxy(locator)));
@@ -729,19 +754,20 @@ public final class ObjectAdapter
     {
 	if(!_deactivated)
 	{
-	    _instance.logger().warning("object adapter `" + _name + "' has not been deactivated");
+	    _instance.initializationData().logger.warning("object adapter `" + _name + "' has not been deactivated");
 	}
-	else if(_instance != null)
+	else if(!_destroyed)
 	{
-	    _instance.logger().warning("object adapter `" + _name + "' deactivation had not been waited for");
+	    _instance.initializationData().logger.warning("object adapter `" + _name +
+	    						  "' has not been destroyed");
 	}
 	else
 	{
-	    IceUtil.Debug.FinalizerAssert(_servantManager == null);
+	    //IceUtil.Debug.FinalizerAssert(_servantManager == null); // Not cleared, it needs to be immutable.
 	    IceUtil.Debug.FinalizerAssert(_communicator == null);
 	    IceUtil.Debug.FinalizerAssert(_incomingConnectionFactories == null);
 	    IceUtil.Debug.FinalizerAssert(_directCount == 0);
-	    IceUtil.Debug.FinalizerAssert(!_waitForDeactivate);
+	    IceUtil.Debug.FinalizerAssert(!_waitForActivate);
 	}
     }
 
@@ -804,8 +830,8 @@ public final class ObjectAdapter
         //
         Connection[] connections = new Connection[0];
         IceInternal.Reference reference =
-	    _instance.referenceFactory().create(ident, new java.util.Hashtable(), facet,
-						IceInternal.Reference.ModeTwoway, false, endpoints, null);
+	    _instance.referenceFactory().create(ident, null, facet, IceInternal.Reference.ModeTwoway, false, endpoints,
+                                                null);
         return _instance.proxyFactory().referenceToProxy(reference);
     }
 
@@ -819,8 +845,7 @@ public final class ObjectAdapter
 	IceInternal.Endpoint[] endpoints = new IceInternal.Endpoint[0];
 	Connection[] connections = new Connection[0];
 	IceInternal.Reference reference =
-	    _instance.referenceFactory().create(ident, new java.util.Hashtable(), facet,
-						IceInternal.Reference.ModeTwoway, false, id, null,
+	    _instance.referenceFactory().create(ident, null, facet, IceInternal.Reference.ModeTwoway, false, id, null,
 						_locatorInfo);
 	return _instance.proxyFactory().referenceToProxy(reference);
     }
@@ -909,19 +934,84 @@ public final class ObjectAdapter
 	return endpoints;
     }
 
+    private void
+    updateLocatorRegistry(IceInternal.LocatorInfo locatorInfo, Ice.ObjectPrx proxy)
+    {
+	if(_id.length() == 0)
+	{
+	    return; // Nothing to update.
+	}
+
+	//
+	// We must get and call on the locator registry outside the
+	// thread synchronization to avoid deadlocks. (we can't make
+	// remote calls within the OA synchronization because the
+	// remote call will indirectly call isLocal() on this OA with
+	// the OA factory locked).
+	//
+	// TODO: This might throw if we can't connect to the
+	// locator. Shall we raise a special exception for the
+	// activate operation instead of a non obvious network
+	// exception?
+	//
+	LocatorRegistryPrx locatorRegistry = locatorInfo != null ? locatorInfo.getLocatorRegistry() : null;
+	if(locatorRegistry == null)
+	{
+	    return;
+	}
+
+	if(_id.length() > 0)
+	{
+	    try
+	    {
+		if(_replicaGroupId.length() == 0)
+		{
+		    locatorRegistry.setAdapterDirectProxy(_id, proxy);
+		}
+		else
+		{
+		    locatorRegistry.setReplicatedAdapterDirectProxy(_id, _replicaGroupId, proxy);
+		}
+	    }
+	    catch(AdapterNotFoundException ex)
+	    {
+		NotRegisteredException ex1 = new NotRegisteredException();
+		ex1.kindOfObject = "object adapter";
+		ex1.id = _id;
+		throw ex1;
+	    }
+	    catch(InvalidReplicaGroupIdException ex)
+	    {
+		NotRegisteredException ex1 = new NotRegisteredException();
+		ex1.kindOfObject = "replica group";
+		ex1.id = _replicaGroupId;
+		throw ex1;
+	    }
+	    catch(AdapterAlreadyActiveException ex)
+	    {
+		ObjectAdapterIdInUseException ex1 = new ObjectAdapterIdInUseException();
+		ex1.id = _id;
+		throw ex1;
+	    }
+	}
+    }    
+
     private boolean _deactivated;
     private IceInternal.Instance _instance;
     private Communicator _communicator;
+    private IceInternal.ObjectAdapterFactory _objectAdapterFactory;
     private IceInternal.ServantManager _servantManager;
-    private boolean _printAdapterReadyDone;
+    private boolean _activateOneOffDone;
     private /*final*/ String _name;
     private /*final*/ String _id;
     private /*final*/ String _replicaGroupId;
     private java.util.Vector _incomingConnectionFactories = new java.util.Vector();
     private java.util.Vector _routerEndpoints = new java.util.Vector();
-    private java.util.Vector _routerInfos = new java.util.Vector();
-    private java.util.Vector _publishedEndpoints;
+    private IceInternal.RouterInfo _routerInfo = null;
+    private java.util.Vector _publishedEndpoints = new java.util.Vector();
     private IceInternal.LocatorInfo _locatorInfo;
     private int _directCount;
-    private boolean _waitForDeactivate;
+    private boolean _waitForActivate;
+    private boolean _destroying;
+    private boolean _destroyed;
 }
