@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -17,13 +17,30 @@ public class ObjectPrxHelperBase implements ObjectPrx
         return _reference.hashCode();
     }
 
+    /**
+     * @deprecated This method has been replaced with ice_getHash.
+     **/
     public final int
     ice_hash()
+    {
+        return ice_getHash();
+    }
+
+    public final int
+    ice_getHash()
     {
         return _reference.hashCode();
     }
 
+    /**
+     * @deprecated This method has been replaced with ice_getCommunicator.
+     **/
     public final Communicator ice_communicator()
+    {
+        return ice_getCommunicator();
+    }
+
+    public final Communicator ice_getCommunicator()
     {
         return _reference.getCommunicator();
     }
@@ -50,10 +67,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
         int __cnt = 0;
         while(true)
         {
+            Connection __connection = null;
             try
             {
 	        __checkTwowayOnly("ice_isA");
-		Connection __connection = ice_connection();
+                __connection = ice_getConnection();
                 IceInternal.Outgoing __og = __connection.getOutgoing(_reference, "ice_isA", OperationMode.Nonmutating,
                                                                      __context);
                 try
@@ -96,11 +114,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__connection, __ex, __cnt);
             }
         }
     }
@@ -117,10 +135,10 @@ public class ObjectPrxHelperBase implements ObjectPrx
         int __cnt = 0;
         while(true)
         {
+            Connection __connection = null;
             try
             {
-	        __checkTwowayOnly("ice_ping");
-		Connection __connection = ice_connection();
+		__connection = ice_getConnection();
                 IceInternal.Outgoing __og = __connection.getOutgoing(_reference, "ice_ping", OperationMode.Nonmutating,
                                                                     __context);
                 try
@@ -154,11 +172,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__connection, __ex, __cnt);
             }
         }
     }
@@ -175,10 +193,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
         int __cnt = 0;
         while(true)
         {
+            Connection __connection = null;
             try
             {
 	        __checkTwowayOnly("ice_ids");
-	        Connection __connection = ice_connection();
+	        __connection = ice_getConnection();
                 IceInternal.Outgoing __og = __connection.getOutgoing(_reference, "ice_ids", OperationMode.Nonmutating,
                                                                     __context);
                 try
@@ -212,11 +231,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__connection, __ex, __cnt);
             }
         }
     }
@@ -233,10 +252,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
         int __cnt = 0;
         while(true)
         {
+            Connection __connection = null;
             try
             {
 	        __checkTwowayOnly("ice_id");
-	        Connection __connection = ice_connection();
+	        __connection = ice_getConnection();
                 IceInternal.Outgoing __og = __connection.getOutgoing(_reference, "ice_id", OperationMode.Nonmutating,
                                                                     __context);
                 try
@@ -270,11 +290,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__connection, __ex, __cnt);
             }
         }
     }
@@ -286,8 +306,12 @@ public class ObjectPrxHelperBase implements ObjectPrx
     }
 
     public final ObjectPrx
-    ice_newIdentity(Identity newIdentity)
+    ice_identity(Identity newIdentity)
     {
+	if(newIdentity.equals(""))
+	{
+	    throw new Ice.IllegalIdentityException();
+	}
         if(newIdentity.equals(_reference.getIdentity()))
         {
             return this;
@@ -300,6 +324,15 @@ public class ObjectPrxHelperBase implements ObjectPrx
         }
     }
 
+    /**
+     * @deprecated This method has been replaced with ice_identity.
+     **/
+    public final ObjectPrx
+    ice_newIdentity(Identity newIdentity)
+    {
+	return ice_identity(newIdentity);
+    }
+
     public final java.util.Hashtable
     ice_getContext()
     {
@@ -307,19 +340,20 @@ public class ObjectPrxHelperBase implements ObjectPrx
     }
 
     public final ObjectPrx
-    ice_newContext(java.util.Hashtable newContext)
+    ice_context(java.util.Hashtable newContext)
     {
 	ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
 	proxy.setup(_reference.changeContext(newContext));
 	return proxy;
     }
 
+    /**
+     * @deprecated This method has been replaced with ice_context.
+     **/
     public final ObjectPrx
-    ice_defaultContext()
+    ice_newContext(java.util.Hashtable newContext)
     {
-	ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-	proxy.setup(_reference.defaultContext());
-	return proxy;
+	return ice_context(newContext);
     }
 
     public final String
@@ -329,7 +363,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     }
 
     public final ObjectPrx
-    ice_newFacet(String newFacet)
+    ice_facet(String newFacet)
     {
         if(newFacet == null)
         {
@@ -348,6 +382,108 @@ public class ObjectPrxHelperBase implements ObjectPrx
         }
     }
 
+    /**
+     * @deprecated This method has been replaced with ice_facet.
+     **/
+    public final ObjectPrx
+    ice_newFacet(String newFacet)
+    {
+	return ice_facet(newFacet);
+    }
+
+    public final String
+    ice_getAdapterId()
+    {
+        return _reference.getAdapterId();
+    }
+
+    public final ObjectPrx
+    ice_adapterId(String newAdapterId)
+    {
+        if(newAdapterId == null)
+        {
+            newAdapterId = "";
+        }
+
+        if(newAdapterId.equals(_reference.getAdapterId()))
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(_reference.changeAdapterId(newAdapterId));
+            return proxy;
+        }
+    }
+
+    public final boolean
+    ice_isSecure()
+    {
+        return _reference.getSecure();
+    }
+
+    public final ObjectPrx
+    ice_secure(boolean newSecure)
+    {
+        if(newSecure == _reference.getSecure())
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(_reference.changeSecure(newSecure));
+            return proxy;
+        }
+    }
+
+    public final Ice.RouterPrx
+    ice_getRouter()
+    {
+	IceInternal.RouterInfo ri = _reference.getRouterInfo();
+	return ri != null ? ri.getRouter() : null;
+    }
+
+    public final ObjectPrx
+    ice_router(Ice.RouterPrx router)
+    {
+        IceInternal.Reference ref = _reference.changeRouter(router);
+        if(ref.equals(_reference))
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(ref);
+            return proxy;
+        }
+    }
+
+    public final Ice.LocatorPrx
+    ice_getLocator()
+    {
+	IceInternal.LocatorInfo ri = _reference.getLocatorInfo();
+	return ri != null ? ri.getLocator() : null;
+    }
+
+    public final ObjectPrx
+    ice_locator(Ice.LocatorPrx locator)
+    {
+        IceInternal.Reference ref = _reference.changeLocator(locator);
+        if(ref.equals(_reference))
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(ref);
+            return proxy;
+        }
+    }
+
     public final ObjectPrx
     ice_twoway()
     {
@@ -414,6 +550,50 @@ public class ObjectPrxHelperBase implements ObjectPrx
 	return _reference.getMode() == IceInternal.Reference.ModeBatchOneway;
     }
 
+    public final ObjectPrx
+    ice_datagram()
+    {
+        IceInternal.Reference ref = _reference.changeMode(IceInternal.Reference.ModeDatagram);
+        if(ref.equals(_reference))
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(ref);
+            return proxy;
+        }
+    }
+
+    public final boolean
+    ice_isDatagram()
+    {
+	return _reference.getMode() == IceInternal.Reference.ModeDatagram;
+    }
+
+    public final ObjectPrx
+    ice_batchDatagram()
+    {
+        IceInternal.Reference ref = _reference.changeMode(IceInternal.Reference.ModeBatchDatagram);
+        if(ref.equals(_reference))
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(ref);
+            return proxy;
+        }
+    }
+
+    public final boolean
+    ice_isBatchDatagram()
+    {
+	return _reference.getMode() == IceInternal.Reference.ModeBatchDatagram;
+    }
+
     public final ObjectPrx
     ice_timeout(int t)
     {
@@ -430,40 +610,17 @@ public class ObjectPrxHelperBase implements ObjectPrx
         }
     }
 
-    public final ObjectPrx
-    ice_router(Ice.RouterPrx router)
+    /**
+     * @deprecated This method has been replaced with ice_getConnection.
+     **/
+    public synchronized final Connection
+    ice_connection()
     {
-        IceInternal.Reference ref = _reference.changeRouter(router);
-        if(ref.equals(_reference))
-        {
-            return this;
-        }
-        else
-        {
-            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-            proxy.setup(ref);
-            return proxy;
-        }
-    }
-
-    public final ObjectPrx
-    ice_locator(Ice.LocatorPrx locator)
-    {
-        IceInternal.Reference ref = _reference.changeLocator(locator);
-        if(ref.equals(_reference))
-        {
-            return this;
-        }
-        else
-        {
-            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-            proxy.setup(ref);
-            return proxy;
-        }
+        return ice_getConnection();
     }
 
     public synchronized final Connection
-    ice_connection()
+    ice_getConnection()
     {
 	if(_connection == null)
 	{
@@ -489,6 +646,12 @@ public class ObjectPrxHelperBase implements ObjectPrx
 	return _connection;
     }
 
+    public synchronized final Connection
+    ice_getCachedConnection()
+    {
+        return _connection;
+    }
+
     public final boolean
     equals(java.lang.Object r)
     {
@@ -531,14 +694,17 @@ public class ObjectPrxHelperBase implements ObjectPrx
     }
 
     public final int
-    __handleException(LocalException ex, int cnt)
+    __handleException(Connection connection, LocalException ex, int cnt)
     {
 	//
 	// Only _connection needs to be mutex protected here.
 	//
 	synchronized(this)
 	{
-	    _connection = null;
+            if(connection == _connection)
+            {
+                _connection = null;
+            }
 	}
 
 	IceInternal.ProxyFactory proxyFactory = _reference.getInstance().proxyFactory();
@@ -557,11 +723,14 @@ public class ObjectPrxHelperBase implements ObjectPrx
     }
 
     public final void
-    __handleExceptionWrapper(IceInternal.LocalExceptionWrapper ex)
+    __handleExceptionWrapper(Connection connection, IceInternal.LocalExceptionWrapper ex)
     {
         synchronized(this)
 	{
-	    _connection = null;
+            if(connection == _connection)
+            {
+                _connection = null;
+            }
 	}
 
 	if(!ex.retry())
@@ -571,17 +740,20 @@ public class ObjectPrxHelperBase implements ObjectPrx
     }
 
     public final int
-    __handleExceptionWrapperRelaxed(IceInternal.LocalExceptionWrapper ex, int cnt)
+    __handleExceptionWrapperRelaxed(Connection connection, IceInternal.LocalExceptionWrapper ex, int cnt)
     {
         if(!ex.retry())
 	{
-	    return __handleException(ex.get(), cnt);
+	    return __handleException(connection, ex.get(), cnt);
 	}
 	else
 	{
 	    synchronized(this)
 	    {
-	        _connection = null;
+                if(connection == _connection)
+                {
+                    _connection = null;
+                }
 	    }
 	    return cnt;
 	}
@@ -603,12 +775,6 @@ public class ObjectPrxHelperBase implements ObjectPrx
 	}
     }
 
-    protected java.util.Hashtable
-    __defaultContext()
-    {
-        return _reference.getContext();
-    }
-
     //
     // Only for use by IceInternal.ProxyFactory
     //
@@ -629,6 +795,9 @@ public class ObjectPrxHelperBase implements ObjectPrx
         _reference = ref;
     }
 
+    protected static final java.util.Hashtable _emptyContext = new java.util.Hashtable();
+
     protected IceInternal.Reference _reference;
     private Connection _connection;
+
 }
