@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -13,8 +13,21 @@ public class BasicStream
 {
     public
     BasicStream(IceInternal.Instance instance)
+    {
+        initialize(instance, false);
+    }
+
+    public
+    BasicStream(IceInternal.Instance instance, boolean unlimited)
+    {
+        initialize(instance, unlimited);
+    }
+
+    private void
+    initialize(IceInternal.Instance instance, boolean unlimited)
     {
         _instance = instance;
+        _unlimited = unlimited;
         allocate(1500);
         _capacity = _buf.capacity();
         _limit = 0;
@@ -114,12 +127,16 @@ public class BasicStream
 	int tmpShrinkCounter = other._shrinkCounter;
 	other._shrinkCounter = _shrinkCounter;
 	_shrinkCounter = tmpShrinkCounter;
+
+        boolean tmpUnlimited = other._unlimited;
+        other._unlimited = _unlimited;
+        _unlimited = tmpUnlimited;
     }
 
     public void
     resize(int total, boolean reading)
     {
-        if(total > _messageSizeMax)
+        if(!_unlimited && total > _messageSizeMax)
         {
             throw new Ice.MemoryLimitException();
         }
@@ -232,7 +249,7 @@ public class BasicStream
 	    //
 	    if(numElements * minSize > bytesLeft) 
 	    {
-		throw new Ice.UnmarshalOutOfBoundsException();
+		Ice.Util.throwUnmarshalOutOfBoundsException();
 	    }
 	}
 	else // Nested sequence
@@ -267,7 +284,7 @@ public class BasicStream
 
 	if(size > bytesLeft)
 	{
-	    throw new Ice.UnmarshalOutOfBoundsException();
+	    Ice.Util.throwUnmarshalOutOfBoundsException();
 	}
     }
 
@@ -282,7 +299,7 @@ public class BasicStream
 	    //
 	    if(numElements * elemSize > bytesLeft) 
 	    {
-		throw new Ice.UnmarshalOutOfBoundsException();
+		Ice.Util.throwUnmarshalOutOfBoundsException();
 	    }
 	}
 	else // Nested sequence
@@ -366,12 +383,12 @@ public class BasicStream
         int sz = readInt();
 	if(sz < 0)
 	{
-	    throw new Ice.NegativeSizeException();
+	    Ice.Util.throwNegativeSizeException();
 	}
 
 	if(sz - 4 > _buf.limit())
 	{
-	    throw new Ice.UnmarshalOutOfBoundsException();
+	    Ice.Util.throwUnmarshalOutOfBoundsException();
 	}
 	_readEncapsStack.sz = sz;
 
@@ -379,12 +396,7 @@ public class BasicStream
         byte eMinor = readByte();
 	if(eMajor != Protocol.encodingMajor || eMinor > Protocol.encodingMinor)
 	{
-	    Ice.UnsupportedEncodingException e = new Ice.UnsupportedEncodingException();
-	    e.badMajor = eMajor < 0 ? eMajor + 256 : eMajor;
-	    e.badMinor = eMinor < 0 ? eMinor + 256 : eMinor;
-	    e.major = Protocol.encodingMajor;
-	    e.minor = Protocol.encodingMinor;
-	    throw e;
+            Ice.Util.throwUnsupportedEncodingException(eMajor, eMinor);
 	}
         _readEncapsStack.encodingMajor = eMajor;
         _readEncapsStack.encodingMinor = eMinor;
@@ -405,27 +417,12 @@ public class BasicStream
         }
         catch(IllegalArgumentException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
         }
 
 	_readEncapsStack = _readEncapsStack.next;
     }
 
-    public void
-    checkReadEncaps()
-    {
-	if(IceUtil.Debug.ASSERT)
-	{
-	    IceUtil.Debug.Assert(_readEncapsStack != null);
-	}
-        int start = _readEncapsStack.start;
-        int sz = _readEncapsStack.sz;
-        if(_buf.position() != start + sz)
-        {
-            throw new Ice.EncapsulationException();
-        }
-    }
-
     public int
     getReadEncapsSize()
     {
@@ -442,7 +439,7 @@ public class BasicStream
         int sz = readInt();
 	if(sz < 0)
 	{
-	    throw new Ice.NegativeSizeException();
+	    Ice.Util.throwNegativeSizeException();
 	}
         try
         {
@@ -450,7 +447,7 @@ public class BasicStream
         }
         catch(IllegalArgumentException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
         }
     }
 
@@ -472,7 +469,7 @@ public class BasicStream
         int sz = readInt();
 	if(sz < 0)
 	{
-	    throw new Ice.NegativeSizeException();
+	    Ice.Util.throwNegativeSizeException();
 	}
         _readSlice = _buf.position();
     }
@@ -486,7 +483,7 @@ public class BasicStream
         int sz = readInt();
 	if(sz < 0)
 	{
-	    throw new Ice.NegativeSizeException();
+	    Ice.Util.throwNegativeSizeException();
 	}
         try
         {
@@ -494,7 +491,7 @@ public class BasicStream
         }
         catch(IllegalArgumentException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
         }
     }
 
@@ -525,7 +522,7 @@ public class BasicStream
 		int v = _buf.getInt();
 		if(v < 0)
 		{
-		    throw new Ice.NegativeSizeException();
+		    Ice.Util.throwNegativeSizeException();
 		}
 		return v;
 	    }
@@ -536,7 +533,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return 0; // Keep the compiler happy.
         }
     }
 
@@ -565,7 +563,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new byte[0]; // Keep the compiler happy.
         }
     }
 
@@ -600,7 +599,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return 0; // Keep the compiler happy.
         }
     }
 
@@ -618,7 +618,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new byte[0]; // Keep the compiler happy.
         }
     }
 
@@ -656,7 +657,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return false; // Keep the compiler happy.
         }
     }
 
@@ -676,7 +678,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new boolean[0]; // Keep the compiler happy.
         }
     }
 
@@ -711,7 +714,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return 0; // Keep the compiler happy.
         }
     }
 
@@ -728,7 +732,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new short[0];
         }
     }
 
@@ -763,7 +768,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return 0; // Keep the compiler happy.
         }
     }
 
@@ -780,7 +786,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new int[0]; // Keep the compiler happy.
         }
     }
 
@@ -815,7 +822,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return 0; // Keep the compiler happy.
         }
     }
 
@@ -832,7 +840,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new long[0]; // Keep the compiler happy.
         }
     }
 
@@ -867,7 +876,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return 0.0f; // Kee the compiler happy.
         }
     }
 
@@ -884,7 +894,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new float[0]; // Keep the compiler happy.
         }
     }
 
@@ -919,7 +930,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return 0.0; // Keep the compiler happy.
         }
     }
 
@@ -936,7 +948,8 @@ public class BasicStream
         }
         catch(ByteBuffer.UnderflowException ex)
         {
-            throw new Ice.UnmarshalOutOfBoundsException();
+            Ice.Util.throwUnmarshalOutOfBoundsException();
+            return new double[0]; // Keep the compiler happy.
         }
     }
 
@@ -1036,7 +1049,8 @@ public class BasicStream
             }
             catch(ByteBuffer.UnderflowException ex)
             {
-                throw new Ice.UnmarshalOutOfBoundsException();
+                Ice.Util.throwUnmarshalOutOfBoundsException();
+                return "";
             }
         }
     }
@@ -1221,7 +1235,7 @@ public class BasicStream
         {
             int oldLimit = _limit;
             _limit += size;
-            if(_limit > _messageSizeMax)
+            if(!_unlimited && _limit > _messageSizeMax)
             {
                 throw new Ice.MemoryLimitException();
             }
@@ -1244,7 +1258,7 @@ public class BasicStream
         }
     }
 
-    private static final class DynamicUserExceptionFactory extends Ice.LocalObjectImpl
+    private static final class DynamicUserExceptionFactory
         implements UserExceptionFactory
     {
         DynamicUserExceptionFactory(Class c)
@@ -1252,11 +1266,6 @@ public class BasicStream
             _class = c;
         }
 
-	DynamicUserExceptionFactory(DynamicUserExceptionFactory source)
-	{
-	    _class = source._class;
-	}
-
         public void
         createAndThrow()
             throws Ice.UserException
@@ -1282,12 +1291,6 @@ public class BasicStream
         {
         }
 
-	public java.lang.Object
-	ice_clone()
-	{
-	    return new DynamicUserExceptionFactory(this);
-	}
-
         private Class _class;
     }
 
@@ -1365,7 +1368,7 @@ public class BasicStream
             if(pos != -1)
             {
                 String topLevelModule = id.substring(2, pos);
-                String pkg = _instance.properties().getProperty("Ice.Package." + topLevelModule);
+                String pkg = _instance.initializationData().properties.getProperty("Ice.Package." + topLevelModule);
                 if(pkg.length() > 0)
                 {
                     c = getConcreteClass(pkg + "." + className);
@@ -1375,7 +1378,7 @@ public class BasicStream
 
         if(c == null)
         {
-            String pkg = _instance.properties().getProperty("Ice.Default.Package");
+            String pkg = _instance.initializationData().properties.getProperty("Ice.Default.Package");
             if(pkg.length() > 0)
             {
                 c = getConcreteClass(pkg + "." + className);
@@ -1422,7 +1425,7 @@ public class BasicStream
         {       
             "abstract", "assert", "boolean", "break", "byte", "case", "catch",
             "char", "checkedCast", "class", "clone", "const", "continue", "default", "do",
-            "double", "else", "equals", "extends", "false", "final", "finalize",
+            "double", "else", "enum", "equals", "extends", "false", "final", "finalize",
             "finally", "float", "for", "getClass", "goto", "hashCode", "if",
             "implements", "import", "instanceof", "int", "interface", "long",
             "native", "new", "notify", "notifyAll", "null", "package", "private",
@@ -1492,6 +1495,14 @@ public class BasicStream
     private void
     reallocate(int size)
     {
+        //
+	// Limit buffer size to MessageSizeMax
+	//
+        if(!_unlimited)
+        {
+	    size = size > _messageSizeMax ? _messageSizeMax : size;
+        }
+
 	ByteBuffer old = _buf;
 	if(IceUtil.Debug.ASSERT)
 	{
@@ -1540,6 +1551,7 @@ public class BasicStream
     private int _writeSlice;
 
     private int _messageSizeMax;
+    private boolean _unlimited;
     private int _shrinkCounter;
 
     private static final class SeqData
