@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -19,7 +19,7 @@ class Twoways
     }
 
     static void
-    twoways(Ice.Communicator communicator, Test.MyClassPrx p)
+    twoways(Ice.Communicator communicator, Ice.InitializationData initData, Test.MyClassPrx p)
     {
         {
             p.opVoid();
@@ -117,9 +117,9 @@ class Twoways
             test(Ice.Util.proxyIdentityAndFacetCompare(c1.value, p) == 0);
             test(Ice.Util.proxyIdentityAndFacetCompare(c2.value, p) != 0);
             test(Ice.Util.proxyIdentityAndFacetCompare(r, p) == 0);
-            test(c1.value.ice_getIdentity().equals(Ice.Util.stringToIdentity("test")));
-            test(c2.value.ice_getIdentity().equals(Ice.Util.stringToIdentity("noSuchIdentity")));
-            test(r.ice_getIdentity().equals(Ice.Util.stringToIdentity("test")));
+            test(c1.value.ice_getIdentity().equals(communicator.stringToIdentity("test")));
+            test(c2.value.ice_getIdentity().equals(communicator.stringToIdentity("noSuchIdentity")));
+            test(r.ice_getIdentity().equals(communicator.stringToIdentity("test")));
             r.opVoid();
             c1.value.opVoid();
             try
@@ -612,7 +612,7 @@ class Twoways
 		test(IceUtil.Hashtable.equals(r, ctx));
 	    }
 	    {
-		Test.MyClassPrx p2 = Test.MyClassPrxHelper.checkedCast(p.ice_newContext(ctx));
+		Test.MyClassPrx p2 = Test.MyClassPrxHelper.checkedCast(p.ice_context(ctx));
 		test(IceUtil.Hashtable.equals(p2.ice_getContext(), ctx));
 		java.util.Hashtable r = p2.opContext();
 		test(IceUtil.Hashtable.equals(r, ctx));
@@ -621,46 +621,19 @@ class Twoways
 	    }
 	    {
 		//
-		// Test that default context is obtained correctly from communicator.
-		//
-		java.util.Hashtable dflt = new java.util.Hashtable();
-		dflt.put("a", "b");
-		communicator.setDefaultContext(dflt);
-		test(!IceUtil.Hashtable.equals(p.opContext(), dflt));
-
-		Test.MyClassPrx p2 = Test.MyClassPrxHelper.uncheckedCast(p.ice_newContext(new java.util.Hashtable()));
-		test(p2.opContext().isEmpty());
-
-		p2 = Test.MyClassPrxHelper.uncheckedCast(p.ice_defaultContext());
-		test(IceUtil.Hashtable.equals(p2.opContext(), dflt));
-
-		communicator.setDefaultContext(new java.util.Hashtable());
-		test(!p2.opContext().isEmpty());
-
-		communicator.setDefaultContext(dflt);
+		// Test proxy contexts
 		String ref = communicator.getProperties().getPropertyWithDefault("Test.Proxy", 
 		    "test:default -p 12010 -t 10000");
 		Test.MyClassPrx c = Test.MyClassPrxHelper.checkedCast(communicator.stringToProxy(ref));
-		test(IceUtil.Hashtable.equals(c.opContext(), dflt)); 
 
-		dflt.put("a", "c");
-		Test.MyClassPrx c2 = Test.MyClassPrxHelper.uncheckedCast(c.ice_newContext(dflt));
-		test(c2.opContext().get("a").equals("c"));
+		java.util.Hashtable dflt = new java.util.Hashtable();
+		dflt.put("a", "b");
+		Test.MyClassPrx c2 = Test.MyClassPrxHelper.uncheckedCast(c.ice_context(dflt));
+		test(c2.opContext().get("a").equals("b"));
 
 		dflt.clear();
-		Test.MyClassPrx c3 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_newContext(dflt));
+		Test.MyClassPrx c3 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_context(dflt));
 		test(c3.opContext().get("a") == null);
-
-		Test.MyClassPrx c4 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_defaultContext());
-		test(c4.opContext().get("a").equals("b"));
-
-		dflt.put("a", "d");
-		communicator.setDefaultContext(dflt);
-
-		Test.MyClassPrx c5 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_defaultContext());
-		test(c5.opContext().get("a").equals("d"));
-
-		communicator.setDefaultContext(new java.util.Hashtable());
 	    }
 	}
 	
