@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -12,7 +12,7 @@ import Test.*;
 public class Client
 {
     public static int
-    run(String[] args, Ice.Communicator communicator, java.io.PrintStream out)
+    run(String[] args, Ice.Communicator communicator, Ice.InitializationData data, java.io.PrintStream out)
     {
         TestIntfPrx test = AllTests.allTests(communicator, false, out);
         test.shutdown();
@@ -28,7 +28,7 @@ public class Client
         try
         {
             communicator = Ice.Util.initialize(args);
-            status = run(args, communicator, System.out);
+            status = run(args, communicator, null, System.out);
         }
         catch(Ice.LocalException ex)
         {
